import sys, os
from PyQt4.QtGui import QMainWindow, QMdiArea, QKeySequence, QAction, QIcon, QDialog,\
     QToolBar, QStatusBar, QMdiSubWindow, QApplication, QFileDialog, QMessageBox, QLabel, \
     QTextEdit, QPushButton, QGridLayout, QLineEdit, QWidget
from PyQt4.QtCore import Qt, QFileInfo
from UIProblem import ProblemWindow
import CooprAgeCore
import cPickle as pickle

class CooprAgeMainWindow(QMainWindow):
    def __init__(self, ownerApplication, ownerCooprApplication):
        self.ownerCooprApplication = ownerCooprApplication
        self.ownerApplication = ownerApplication
        QMainWindow.__init__(self)
        self.mdiarea = QMdiArea()
        self.setCentralWidget(self.mdiarea)
        self.setObjectName("CooprAge Main Window")
        self.setWindowIcon(QIcon('images/small_coopr_logo.png'))
        self.initUI()
        self.read_preferences()
        

#        self.ownerCooprApplication.addNewProblem()
        self.update()

    def closeEvent(self, event):
        for window in self.mdiarea.subWindowList():
            problemWindow = window.widget()
            self.maybeSave(problemWindow, problemWindow.problem, False)

        self.write_preferences()
        event.accept()

    def write_preferences(self):
        prefobj = dict()
        prefobj['uistate'] = self.saveState()
        if self.ownerCooprApplication.pyomoExe is not None:
            prefobj['pyomo_exe'] = self.ownerCooprApplication.pyomoExe
	if self.ownerCooprApplication.fontSize is not None:
	    prefobj['font_size'] = self.ownerCooprApplication.fontSize
            
        preffile = open('UIPrefs.ini','w+')
        pickle.dump(prefobj, preffile)
        preffile.close()

    def read_preferences(self):
        if os.path.exists('UIPrefs.ini'):
            prefile = None
            try:
                preffile = open('UIPrefs.ini','r')
                prefobj = pickle.load(preffile)
                if 'uistate' in prefobj:
                    self.restoreState(prefobj['uistate'])
                if 'pyomo_exe' in prefobj:
                    self.ownerCooprApplication.pyomoExe = prefobj['pyomo_exe']
		if 'font_size' in prefobj:
		    self.ownerCooprApplication.fontSize = prefobj['font_size']
                
                preffile.close()
                
            except:
                if preffile is not None:
                    preffile.close()
                self.ownerCooprApplication.pyomoExe = None
            
    def initUI(self):
        """Initializes the GUI components (window title, menus, toolbars, etc)."""
        self.setWindowTitle('Coopr.AGE (Advanced Graphical Environment for Pyomo)')
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setUnifiedTitleAndToolBarOnMac(True)

        toolbar = QToolBar()
        toolbar.setObjectName('MainToolBar')
        toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
        toolbar.addAction(QAction(QIcon('images/document-new-7.png'), \
                                  "&New", self,  \
                                  statusTip="Create a new Coopr problem", triggered=self.newCooprProblem))
        toolbar.addAction(QAction(QIcon('images/document-open-4.png'), \
                                  "&Open...", self, \
                                  statusTip="Open an existing Coopr problem", triggered=self.openCooprProblem))
        toolbar.addAction(QAction(QIcon('images/document-save-2.png'), \
                                  "&Save", self, shortcut=QKeySequence.Save, \
                                  statusTip="Save the Coopr problem to disk", triggered=self.saveActiveCooprProblem))
        toolbar.addAction(QAction(QIcon('images/document-save-as-2.png'), \
                                  "Save &As...", self, shortcut=QKeySequence.SaveAs, \
                                  statusTip="Save the Coopr problem under a new name", \
                                  triggered=self.saveAsActiveCooprProblem))
        toolbar.addAction(QAction(QIcon('images/configure.png'), \
                                  "Preferences...", self, shortcut=QKeySequence.SaveAs, \
                                  statusTip="Set preferences for Coopr.AGE", \
                                  triggered=self.openCooprPreferences))

        self.addToolBar(Qt.TopToolBarArea, toolbar)
	
    def browsePyomoExe(self):
	# we need to ask the user for the pyomo executable
        pyomoPath = QFileDialog.getOpenFileName(self, "Find the pyomo script", filter="pyomo executable (*)")
        if pyomoPath != "":
            canonicalFilePath = QFileInfo(pyomoPath).canonicalFilePath()
            return canonicalFilePath
	return None
	    
    def newCooprProblem(self):
        self.ownerCooprApplication.addNewProblem()
        self.update()

    def openCooprProblem(self):
        filePath = QFileDialog.getOpenFileName(self, filter="Coopr Problem File [*.cpf] (*.cpf);;All files [*] (*)")
        if filePath != "":
            canonicalFilePath = QFileInfo(filePath).canonicalFilePath()
            name = QFileInfo(canonicalFilePath).fileName()
            self.ownerCooprApplication.addPickledProblem(canonicalFilePath, name)
            self.update()
            
    def maybeSave(self,problemWindow, problem, allow_cancel):
        if allow_cancel == True:
            choice = QMessageBox.warning(self, 'Save Document?', 'Do you wish to save ' + problem.name + '?', QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel)
        else:
            choice = QMessageBox.warning(self, 'Save Document?', 'Do you wish to save ' + problem.name + '?', QMessageBox.Save | QMessageBox.Discard)
            
        if choice == QMessageBox.Save:
            return self.saveCooprProblem(problemWindow, problem)
        return choice
        
    def saveActiveCooprProblem(self):
        window = self.mdiarea.activeSubWindow()
        if not isinstance(window, ProblemWindow):
            window = window.widget()
        assert(isinstance(window, ProblemWindow))
        problem = window.problem
        return self.saveCooprProblem(window, problem)
        
    def saveCooprProblem(self, problemWindow, problem):
        if problem.filename is None:
            return self.saveAsCooprProblem(problem)

        filename = problem.filename
        name = QFileInfo(filename).fileName()
        problemWindow.pushEditorData()
        self.ownerCooprApplication.pickleProblem(problem, filename, name)
        self.update()
        return QMessageBox.Ok

    def saveAsActiveCooprProblem(self):
        window = self.mdiarea.activeSubWindow()
        if not isinstance(window, ProblemWindow):
            window = window.widget()
        assert(isinstance(window, ProblemWindow))
        problem = window.problem
        return self.saveAsCooprProblem(window, problem)
    
    def saveAsCooprProblem(self, problemWindow, problem):
        if problem.filename is None:
            filePath = QFileDialog.getSaveFileName(self, "Save As", "", filter="Coopr Problem File [*.cpf] (*.cpf)")
        else:
            filePath = QFileDialog.getSaveFileName(self, "Save As", problem.filename, filter="Coopr Problem File [*.cpf] (*.cpf)")

        if filePath != "":
            name = QFileInfo(filePath).fileName()
            problemWindow.pushEditorData()
            self.ownerCooprApplication.pickleProblem(problem, filePath, name)
            self.update()
            return QMessageBox.Ok
        else:
            return QMessageBox.Cancel
        
    def openCooprPreferences(self):
        dialog = CooprAgeSettingsDialog(self, self.ownerCooprApplication)
        dialog.show()

    def update(self):
        for problem in self.ownerCooprApplication.problems:
            found = False
            for window in self.mdiarea.subWindowList():
                if not isinstance(window, ProblemWindow):
                    window = window.widget()
                assert(isinstance(window, ProblemWindow))
                if window.problem == problem:
                    found = True
                    break

            if found == False:
                win = ProblemWindow(self, self.ownerCooprApplication, problem)
                self.mdiarea.addSubWindow(win)
                win.show()

        for window in self.mdiarea.subWindowList():
            if not isinstance(window, ProblemWindow):
                window = window.widget()
            assert(isinstance(window, ProblemWindow))
            window.update()
            
class CooprAgeSettingsDialog(QDialog):
    def __init__(self, ownerWindow, ownerCooprApplication):
        QDialog.__init__(self, ownerWindow)
	self.ownerWindow = ownerWindow
        self.ownerCooprApplication = ownerCooprApplication
        self.setModal(True)
        self.initUI()

    def initUI(self):
        self.setWindowTitle("Coopr.AGE Settings")

        self.layout = QGridLayout(self)
        self.layout.setVerticalSpacing(6)
	self.layout.setColumnMinimumWidth(1,300)
        self.layout.setColumnStretch(1,2)

        self.pyomoExeLabel = QLabel("Pyomo Executable:", self)
        self.pyomoExeEdit = QLineEdit(self)
        self.pyomoExeButton = QPushButton('Browse...', self)
        self.layout.addWidget(self.pyomoExeLabel,0,0)
        self.layout.addWidget(self.pyomoExeEdit,0,1)
        self.layout.addWidget(self.pyomoExeButton,0,2)
        self.fontSizeLabel = QLabel('Editor Font Size: ', self)
        self.fontSizeEdit = QLineEdit(self)
        self.layout.addWidget(self.fontSizeLabel,1,0)
        self.layout.addWidget(self.fontSizeEdit,1,1)

        buttoncontainer = QWidget(self)
        buttonlayout = QGridLayout(buttoncontainer)
        self.acceptButton = QPushButton("OK", buttoncontainer)
        buttonlayout.addWidget(self.acceptButton,0,0)

        self.cancelButton = QPushButton("Cancel", buttoncontainer)
        buttonlayout.addWidget(self.cancelButton,0,1)
        buttoncontainer.setLayout(buttonlayout)
        self.layout.addWidget(buttoncontainer,2,0,1,3)

        self.setLayout(self.layout)

        self.pyomoExeButton.clicked.connect(self.browsePyomoExeButton)
        self.acceptButton.clicked.connect(self.accept)
        self.acceptButton.setDefault(True)
        self.cancelButton.clicked.connect(self.closeDialog)

        self.update()

    def browsePyomoExeButton(self):
	text = self.ownerWindow.browsePyomoExe()
	if text is not None:
	    self.pyomoExeEdit.setText(text)
        
    def update(self):
        if self.ownerCooprApplication.pyomoExe is not None:
            self.pyomoExeEdit.setText(str(self.ownerCooprApplication.pyomoExe))
        if self.ownerCooprApplication.fontSize is not None:
            self.fontSizeEdit.setText(str(self.ownerCooprApplication.fontSize))
    
    def accept(self):
        self.ownerCooprApplication.pyomoExe = None
        if self.pyomoExeEdit.text() != '':
            self.ownerCooprApplication.pyomoExe = str(self.pyomoExeEdit.text())

        if self.fontSizeEdit.text() != '':
            self.ownerCooprApplication.fontSize = int(self.fontSizeEdit.text())

	self.ownerWindow.update()
        self.close()

    def closeDialog(self):
        self.close()


if __name__ == "__main__":
    # when one "exectutes" a python program, it's __name__ is
    # __main__, otherwise it's name will be the module name
    app = QApplication(sys.argv)
    cooprage_app = CooprAgeCore.CooprAgeApplication()
    mainWindow = CooprAgeMainWindow(app, cooprage_app)
    mainWindow.show()
    app.exec_()



import sys, os
#from PyQt4.QtGui import QTextDocument, QPlainTextDocumentLayout
#from PyQt4.QtCore import QFileInfo, QTextStream, QFile
import pickle
import tempfile

class CooprAgeApplication(object):
    def __init__(self):
        self.problems = list()
        self.pyomoExe = None
        self.fontSize = 12

    def addNewProblem(self):
        problem = CooprAgeProblem(self)
        self.problems.append(problem)
        return problem
    
    def closeProblem(self, problem):
        if problem in self.problems:
            self.problems.remove(problem)
    
    def pickleProblem(self, problem, filename, name):
        file = open(filename,'w+')
        problem.filename = filename
        problem.name = name
        pickle.dump(problem, file)
        file.close()
        
    def addPickledProblem(self, filename, name):
        file = open(filename,'r')
        problem = pickle.load(file)
        problem.filename = filename
        problem.name = name
        self.problems.append(problem)
        file.close()
        return problem

class CooprAgeRunOptions(object):
    def __init__(self):
        self.disableGC = False
        self.timeLimit = 3000
        self.streamOutput = False
        self.verboseOutput = False
        self.quietOutput = False
        self.summaryOutput = False
        self.printSolverLog = False
        self.saveLogFile = False
        self.logFileName = ''
        self.solver = 'glpk'
        self.solverOptions = ''
        self.solverMipGap = 0
        self.keepTempFiles = False

class CooprAgeProblem(object):
    sequenceNumber = 1
    
    def __init__(self, ownerApplication):
        self.ownerApplication_ = ownerApplication
        
        self.name = 'Problem-'+str(CooprAgeProblem.sequenceNumber)
        self.filename = None
        
        self.inputDocuments = list()
        self.inputDocumentStates = list()
        self.outputDocuments = list()
                                
        CooprAgeProblem.sequenceNumber = CooprAgeProblem.sequenceNumber + 1
        self.newInputDocument('model', 'model')
        self.newInputDocument('data', 'data')
        self.newInputDocument('inactive','notes')
        
        # specify the run options
        self.run_options = dict()
        self.run_options["help"] = None
        self.run_options["solver"] = None
        self.run_options["solver-options"] = None
        self.run_options["solver-mipgap"] = None
        self.run_options["timelimit"] = None
        self.run_options["log"] = None
        self.run_options["logfile"] = None
        self.run_options["quiet"] = None
        self.run_options["summary"] = None
        self.run_options["stream-output"] = None
        self.run_options["verbose"] = None
        self.run_options["warning"] = None
        
        self.run_options["__UserCmd__"] = None
        
    def getDocumentState(self, cooprdoc):
        if cooprdoc in self.inputDocuments:
            docidx = self.inputDocuments.index(cooprdoc)
            return self.inputDocumentStates[docidx]
        return None
    
    def setDocumentState(self, cooprdoc, state):
        if cooprdoc in self.inputDocuments:
            assert(state == 'model' or state == 'data' or state == 'inactive')
            docidx = self.inputDocuments.index(cooprdoc)
            self.inputDocumentStates[docidx] = state
    
    def NameUnique(self, name):
        docs = list()
        docs.extend(self.inputDocuments)
        docs.extend(self.outputDocuments)
        for doc in docs:
            if doc.GetName() == name:
                return False
        return True
        
    def getUniqueName(self, prefix):
        for i in xrange(1,32767):
            name = prefix + str(i)
            if self.NameUnique(name):
                return name
        return None
    
    def newInputDocument(self, state, name=None):
        if name is None:
            name = 'Doc'
        cooprdoc = CooprAgeDocument(self,name)
        self.inputDocuments.append(cooprdoc)
        self.inputDocumentStates.append(state)
        return cooprdoc

    def newOutputDocument(self, name=None):
        if name is None:
            name = 'Run'
        cooprdoc = CooprAgeDocument(self,name)
        self.outputDocuments.append(cooprdoc)
        return cooprdoc
            
    def removeInputDocument(self, cooprdoc):
        if cooprdoc in self.inputDocuments:
            self.inputDocuments.remove(cooprdoc)

    def removeOutputDocument(self, cooprdoc):
        if cooprdoc in self.outputDocuments:
            self.outputDocuments.remove(cooprdoc)
    
    def getInputDocuments(self):
        return self.inputDocuments
    
    def getOutputDocuments(self):
        return self.outputDocuments

    def getDocument(self, name):
        for doc in self.inputDocuments:
            if doc.GetName() == name:
                return doc

        for doc in self.outputDocuments:
            if doc.GetName() == name:
                return doc
        
    def prepareForRun(self):
        # build up the command line...
        cmd = ""
        for (key,value) in self.run_options.items():
            if value is not None and key != "__UserCmd__":
                if value == True:
                    cmd += "--" + key + " "
                elif key == "solver-options":
                    cmd += "--" + key + "=\"" + value + "\" "
                else:
                    cmd += "--" + key + "=" + value + " "
        
        usercmd = self.run_options["__UserCmd__"]
        if usercmd is not None:
            cmd += usercmd + " "

        files = list()
        # need to save temp file for the model
        foundmodel = False
        for modelfile in self.inputDocuments:
            if self.getDocumentState(modelfile) == 'model':
                (mfile, mfilename) = tempfile.mkstemp(suffix='.py', prefix='coopr_model', dir='./workspace', text=True)
                os.write(mfile,str(modelfile.text))
                os.close(mfile)
                cmd = cmd + mfilename + " ";
                files.append(mfilename)
                break
        
        for datfile in self.inputDocuments:
            if self.getDocumentState(datfile) == 'data':
                (dfile, dfilename) = tempfile.mkstemp(suffix='.dat', prefix='coopr_dat', dir='./workspace', text=True)
                cmd = cmd + dfilename + " ";
                os.write(dfile,str(datfile.text))
                os.close(dfile)
                files.append(dfilename)
                cmd += dfilename + " "
                
        return (cmd, files)
    
    def cleanupAfterRun(self, files):
        for file in files:
            os.remove(file)
        
class CooprAgeDocument(object):
    sequenceNumber = 1
    
    def __init__(self, ownerProblem, name=None):
        self.ownerProblem = ownerProblem
        self.SetName(name)
        self.text = str()

    def SetName(self, name):
        if name is not None and self.ownerProblem.NameUnique(name) and name is not 'Run' and name is not 'Doc':
            self.name = name
        elif name is None:
            self.name = self.ownerProblem.getUniqueName('Doc-')
        else:
            self.name = self.ownerProblem.getUniqueName(name + '-')
    
    def GetName(self):
        return self.name



#import sys, os
from PyQt4.QtGui import QPlainTextEdit, QGridLayout, QWidget, QTabWidget, QFont, QPlainTextEdit, QAction, QIcon
from PyQt4.Qt import QSize, Qt
import PyomoSyntax

class ProblemEditor(QWidget):
    def __init__(self, ownerWidget):
        QWidget.__init__(self, ownerWidget)
        self.ownerProblemWindow = ownerWidget
        self.layout = QGridLayout(self)
        self.tabWidget = QTabWidget(self)
#        self.tabWidget.runAction = QAction(QIcon('images/remove.png'), \
#                                 "Close", self, \
#                                 statusTip="Close Document", triggered=self.closeActiveDocument)
#        self.tabWidget.addAction(self.tabWidget.runAction)
#        self.tabWidget.setContextMenuPolicy(Qt.ActionsContextMenu)
        self.tabWidget.setTabsClosable(True)
        self.tabWidget.tabCloseRequested.connect(self.closeTab)
        self.layout.addWidget(self.tabWidget)
        self.setLayout(self.layout)
        self.textEdits = list()
        if self.ownerProblemWindow.ownerCooprApplication.fontSize is not None:
            self.font = QFont("Courier", int(self.ownerProblemWindow.ownerCooprApplication.fontSize))
        else:
            self.font = QFont('Courier', 10)
        self.font.setFixedPitch(True)
        self.font.setStyleHint(QFont.Courier, QFont.PreferAntialias)
        
    def updateFont(self):
        if self.ownerProblemWindow.ownerCooprApplication.fontSize is not None:
            self.font = QFont("Courier", self.ownerProblemWindow.ownerCooprApplication.fontSize)
        else:
            self.font = QFont('Courier', 10)
        self.update()
        
    def sizeHint(self):
        return QSize(500,200)
    
    def closeTab(self, idx):
        name = self.tabWidget.tabText(idx)
        self.ownerProblemWindow.pushEditorData(name)
        self.tabWidget.removeTab(idx)        
        
    def addDocument(self, name, initialText):
        # check if the document is already open
        tabidx = self.getTabIdx(name)
        textEdit = None
        if tabidx is None:
            textEdit = QPlainTextEdit()
            self.textEdits.append(textEdit)
            textEdit.document().setPlainText(initialText)
            textEdit.setFont(self.font)
            textEdit.syntaxHighlighter = PyomoSyntax.Python(textEdit.document())
            textEdit.document().modificationChanged.connect(self._modificationChanged)
            self.tabWidget.addTab(textEdit, name)
            textEdit.document().setModified(False)
            self.tabWidget.setCurrentIndex(self.tabWidget.count()-1)
        else:
            textEdit = self.tabWidget.widget(tabidx)
            self.tabWidget.setCurrentIndex(tabidx)
            
        return textEdit

    def closeDocument(self, name):
        tabidx = self.getTabIdx(name)
        if tabidx is not None:
            self.tabWidget.removeTab(tabidx)

    def changeName(self, oldname, newname):
        tabidx = self.getTabIdx(oldname)
        if tabidx is not None:
            self.tabWidget.setTabText(newname)
            self.update()

    def addText(self, name, text):
        tabidx = self.getTabIdx(name)
        if tabidx is not None:
            textEdit = self.tabWidget.widget(tabidx)
            textEdit.appendPlainText(text)

    def setText(self, name, text):
        tabidx = self.getTabIdx(name)
        if tabidx is not None:
            textEdit = self.tabWidget.widget(i)
            textEdit.document().setPlainText(text)

    def getTextEdit(self, name):
        tabidx = self.getTabIdx(name)
        if tabidx is not None:
            return self.tabWidget.widget(tabidx)
        return None

    def getTabIdx(self, name):
        # find the page that corresponds to the document and close
        N = self.tabWidget.count()
        tabWidget = self.tabWidget
        tabidx = None
        for i in range(0,N):
            tabtext = tabWidget.tabText(i)
            if tabtext == name or tabtext == name + '*':
                # found it
                tabidx = i
                break

        return tabidx

    def getDocumentNames(self):
        names = list()
        N = self.tabWidget.count()
        tabWidget = self.tabWidget
        for i in range(0,N):
            tabText = tabWidget.tabText(i)
            # to do: remove '*' if necessary
            names.append(tabText)

        return names
    
    def update(self):
        N = self.tabWidget.count()
        tabsToRemove = list()
        for i in range(0,N):
            widget = self.tabWidget.widget(i)
            widget.setFont(self.font)
            tabname = self.tabWidget.tabText(i)
            if self.ownerProblemWindow.problem.getDocument(tabname) is None:
                tabsToRemove.append(i)                
        
        for i in tabsToRemove:
            self.tabWidget.removeTab(i)

        return
        N = self.tabWidget.count()
        for i in range(0,N):
            textEdit = self.tabWidget.widget(i)
            tabname = self.tabWidget.tabText(i)
            if textEdit.document().isModified() == True and not str(tabname).endswith('*'):
                self.tabWidget.setTabText(i, tabname + '*')
            elif textEdit.document().isModified() == False and str(tabname).endswith('*'):
                # to do: fix when we get internet
                self.tabWidget.setTabText(tabname + 'x')

    def getText(self, name):
        tabidx = self.getTabIdx(name)
        if tabidx is not None:
            textEdit = self.tabWidget.widget(i)
            return textEdit.document().toPlainText()
        return None

    def _modificationChanged(self):
        self.update()
